clear all ;% clear all variables
close all

%============== Define global variables ===================================================
global BitRate Bits TimeWindow SampleRate DataLength samplesperbit t_range f_sym Ref_Frequency

BitRate = 1e9;  
Bits2Power = 10;
Bits = 2^Bits2Power;
TimeWindow = Bits/BitRate;
SampleRate = BitRate*2^10;
DataLength = TimeWindow*SampleRate;
Ref_Frequency = 193.1e12; %[Hz] Reference optical frequency
samplesperbit = SampleRate/BitRate; % data number per bit

% time, optical frequency range
t_range = 0:1/SampleRate:1/SampleRate*(DataLength-1);
f_sym = SampleRate*(-DataLength/2:(DataLength/2-1))/DataLength; %

%==========================================================================================
%=================== Define Simulation Blocks =============================================
% SM Laser
UnmodulatedPower = 2e-3; %[W] 
Linewidth = 10e6; %[Hz]
Laser1 = LaserCW(UnmodulatedPower,Linewidth);
%------------- Data Source -------------------%
PreSpaces = 2;  %number of leading zero bits
PostSpaces = 2; %number of trailing zero bits
info_bits = infobits_PRBS(PreSpaces,PostSpaces);

E_data = NRZCoder(info_bits);

ERiseTime = 1/4/BitRate; %  In the form of k/BitRate, k should be less than (1/2)*RZRatio if RZCoder used   
E_elec = RiseTimeAdjust(E_data,ERiseTime);

ER = 30;% Extinction Ratio [dB]
ModulationIndex = 1-10^(-ER/10); % Note)Extinction Ratio [dB] = -10log10(1-ModulationIndex)
mod_out = ModulatorAmplitude(Laser1,E_elec,ModulationIndex);

%---------------- Transmission -------------------------------------
loss = 40; % [dB]
%loss = input('What is the Attenuation in [dB]?  '); % 20 ~ 30 [dB]
mod_att = Attenuator(mod_out,loss);

%--------------- Receiver (EDFA+PIN) ------------------------------------------
G_Target = 20; %
EDFA_out = EDFA_gain(mod_att,G_Target);
OptFilter_out = OpticalFilter_Bessel(EDFA_out,0,5*BitRate); % 

Responsivity = 1.0;% [A/W], [0,2]
ThermalNoiseDen = 1e-22;% [A^2/Hz] 
DarkCurrent = 0;% [A], [0,10e-6]
ShotNoiseFactor = 0;%   (1 if shot noise included, 0 if not)

y_PIN = PhotoDiode_PIN(OptFilter_out,Responsivity,ThermalNoiseDen,DarkCurrent,ShotNoiseFactor); 

FilterOrder = 3;
BW3dB = 0.75*BitRate;

y_PIN_LP = E_FilterLPBessel(y_PIN,FilterOrder,BW3dB);
%--------------- Ouput Observation & BER Calculation --------------------------------------

EyePlotter(y_PIN_LP)
title('Optical Preamp. Receiver Output')
BER_PIN = qnber_noISI(y_PIN_LP,info_bits,0.5, 0.5);


% Optical Filter Bandwidth Sweep

BW_opt = (1:10)*BitRate;

for k = 1:length(BW_opt);
    
    OptFilter_out = OpticalFilter_Bessel(EDFA_out,0,BW_opt(k),5); % 
    y_PIN_k = PhotoDiode_PIN(OptFilter_out,Responsivity,ThermalNoiseDen,DarkCurrent,ShotNoiseFactor); 
    y_PIN_LPk = E_FilterLPBessel(y_PIN_k,FilterOrder,BW3dB);
    BER_opt(k) = qnber_noISI(y_PIN_LPk,info_bits,0.5, 0.5);
    
end

figure(2)
semilogy(BW_opt,BER_opt,'-*')
%xlabel('뿪 [Hz]'), ylabel('BER')
xlabel('B_O [Hz]'), ylabel('BER')

% EDFA noise figure sweep

opt_NF=3:8;

for kk = 1:length(opt_NF);
    
    EDFA_out = EDFA_gain(mod_att,G_Target,'Yes',opt_NF(kk));
    OptFilter_out = OpticalFilter_Bessel(EDFA_out,0,5*BitRate); % 
    y_PIN_kk = PhotoDiode_PIN(OptFilter_out,Responsivity,ThermalNoiseDen,DarkCurrent,ShotNoiseFactor); 
    y_PIN_LPkk = E_FilterLPBessel(y_PIN_kk,FilterOrder,BW3dB);
    BER_NF(kk) = qnber_noISI(y_PIN_LPkk,info_bits,0.5, 0.5);
    
end

figure(3)
semilogy(opt_NF,BER_NF,'-*')
%xlabel('[dB]'), ylabel('BER')
xlabel('NF[dB]'), ylabel('BER')

